@@ -3,7 +3,7 @@ import asyncio
 import os
 import shutil
 from types import TracebackType
-from typing import Any, Callable, Dict, Generator, IO, Optional, Tuple, Type, Union
+from typing import Any, AsyncGenerator, Callable, Coroutine, Dict, Generator, IO, Iterable, Optional, Tuple, Type, Union
 from aiofiles.threadpool.binary import AsyncBufferedIOBase
 
 import aiohttp, aiofiles
@@ -11,7 +11,6 @@ from aiohttp.client import DEFAULT_TIMEOUT
 
 from aiodav.utils import WebDavXmlUtils
 from aiodav.urn import Urn
-from aiodav.resource import Resource
 from aiodav.exceptions import *
 
 log = logging.getLogger(__name__)
@@ -40,6 +39,18 @@ class Client(object):
         timeout (``int``, *optional*):
             Time limit for all operations. Default is 300 (5 min).
 
+        insecure (``bool``, *optional*):
+            Allow insecure server connections when using SSL. Default is False.
+
+        proxy (``str``, *optional*):
+            Use this proxy with format [protocol://]host[:port]. Ex: http://localhost:3128
+
+        proxy_user (``str``, *optional*):
+            Set a user to use in proxy authentication.
+
+        proxy_password (``str``, *optional*):
+            Set a password to use in proxy authentication.
+
         chunk_size (``int``, *optional*):
             Size of buffer used to transfer data from/to server. This data will be 
             loaded in memory. This parameter afect the progress callback in 
@@ -48,9 +59,6 @@ class Client(object):
 
     ROOT = '/'
 
-    # controls whether to verify the server's TLS certificate or not
-    VERIFY = True
-
     # HTTP headers for different actions
     DEFAULT_HTTP_HEADER = {
         'list': ["Accept: */*", "Depth: 1"],
@@ -94,6 +102,10 @@ class Client(object):
         root: Optional[str] = ROOT,
         timeout: Optional[int] = None,
         chunk_size: Optional[int] = None,
+        proxy: Optional[str] = None,
+        proxy_user: Optional[str] = None,
+        proxy_password: Optional[str] = None,
+        insecure: Optional[bool] = False,
         loop: Optional[asyncio.AbstractEventLoop] = None,
         **kwargs: Any
     ) -> None:
@@ -101,6 +113,10 @@ class Client(object):
         self._token = token
         self._root = (Urn(root).quote() if root else '').rstrip(Urn.separate)
         self._chunk_size = chunk_size if chunk_size else 65536
+        self._proxy = proxy
+        self._proxy_auth = aiohttp.BasicAuth(proxy_user, proxy_password) if (
+                proxy_user and proxy_password) else None
+        self._insecure = insecure
         self.session = aiohttp.ClientSession(
             loop = loop,
             timeout = aiohttp.ClientTimeout(total=timeout) if timeout else DEFAULT_TIMEOUT,
@@ -111,8 +127,8 @@ class Client(object):
     def _get_headers(
         self, 
         action: str, 
-        headers_ext: Optional[dict[str, str]] = None
-    ) -> dict[str, str]:
+        headers_ext: Optional[Dict[str, str]] = None
+    ) -> Dict[str, str]:
         headers = None
         if action in Client.DEFAULT_HTTP_HEADER:
             headers = dict([map(lambda s: s.strip(), x.split(':', 1)) for x in Client.DEFAULT_HTTP_HEADER[action]])
@@ -144,7 +160,7 @@ class Client(object):
         action: str,
         path: Union[str, "os.PathLike[str]"],
         data: Optional[Any] = None,
-        headers_ext: Optional[dict[str, str]] = None
+        headers_ext: Optional[Dict[str, str]] = None
     ) -> aiohttp.ClientResponse:
         try:
             if self.session.auth:
@@ -155,8 +171,10 @@ class Client(object):
                 url = self._get_url(path),
                 headers = self._get_headers(action, headers_ext),
                 data = data,
+                proxy = self._proxy,
+                proxy_auth = self._proxy_auth,
                 # chunked = self._chunk_size,
-                # ssl = 
+                ssl = False if self._insecure else None
             )
 
             if response.status == 507:
@@ -205,7 +223,7 @@ class Client(object):
         self,
         path: Optional[Union[str, "os.PathLike[str]"]] = ROOT,
         get_info: Optional[bool] = False
-    ) -> Union[list[str], list[dict[str, str]]]:
+    ) -> Union[list[str], list[Dict[str, str]]]:
         """
         Returns list of nested files and directories for remote WebDAV directory by path.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFIN
@@ -218,7 +236,7 @@ class Client(object):
                 Set true to get more information like cmd 'ls -l'.
 
         Returns:
-            List of :obj:`str` | List of :obj:`dict[str, str]`: On success, if get_info=False it returns 
+            List of :obj:`str` | List of :obj:`Dict[str, str]`: On success, if get_info=False it returns 
                 list of nested file or directory names, otherwise it returns list of information, the 
                 information is a dictionary and it values with following keys:
 
@@ -348,7 +366,7 @@ class Client(object):
     async def info(
         self, 
         path: Union[str, "os.PathLike[str]"]
-    ) -> dict[str, str]:
+    ) -> Dict[str, str]:
         """
         Gets information about resource on WebDAV.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFIND
@@ -358,7 +376,7 @@ class Client(object):
                 Path to remote resource.
 
         Returns:
-            :obj:`dict[str, str]`: a dictionary of information attributes and them values with following keys:
+            :obj:`Dict[str, str]`: a dictionary of information attributes and them values with following keys:
 
                 `created`: date of resource creation,
                 `name`: name of resource,
@@ -761,7 +779,7 @@ class Client(object):
     async def upload_to(
         self,
         path: Union[str, "os.PathLike[str]"],
-        buffer: IO,
+        buffer: Union[IO, AsyncGenerator[bytes, None]],
         buffer_size: Optional[int] = None,
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
         progress_args: Optional[Tuple] = ()
@@ -775,7 +793,8 @@ class Client(object):
                 The path to remote resource
 
             buffer (``IO``)
-                IO like object to read the data.
+                IO like object to read the data or a asynchronous generator to get buffer data.
+                In order do you select use a async generator `progress` callback cannot be called.
 
             progress (``callable``, *optional*):
                 Pass a callback function to view the file transmission progress.
@@ -816,31 +835,31 @@ class Client(object):
             raise OptionNotValid(name="path", value=path)
 
         if not (await self.exists(urn.parent())):
-            raise RemoteParentNotFound(urn.path())
+            raise RemoteParentNotFound(urn.path())      
         
-        async def file_sender(buff: IO):
-            current = 0
-            
-            if asyncio.iscoroutinefunction(progress):
-                await progress(current, buffer_size, *progress_args)
-            else:
-                progress(current, buffer_size, *progress_args)
-
-            while current < buffer_size:
-                chunk = await buffer.read(self._chunk_size) if isinstance(buffer, AsyncBufferedIOBase) \
-                    else buffer.read(self._chunk_size)
-                if not chunk:
-                    break
-                
-                current += len(chunk)
+        if callable(progress) and not asyncio.iscoroutinefunction(buffer):
+            async def file_sender(buff: IO):
+                current = 0
                 
                 if asyncio.iscoroutinefunction(progress):
                     await progress(current, buffer_size, *progress_args)
                 else:
                     progress(current, buffer_size, *progress_args)
-                yield chunk                
 
-        if callable(progress):
+                while current < buffer_size:
+                    chunk = await buffer.read(self._chunk_size) if isinstance(buffer, AsyncBufferedIOBase) \
+                        else buffer.read(self._chunk_size)
+                    if not chunk:
+                        break
+                    
+                    current += len(chunk)
+                    
+                    if asyncio.iscoroutinefunction(progress):
+                        await progress(current, buffer_size, *progress_args)
+                    else:
+                        progress(current, buffer_size, *progress_args)
+                    yield chunk                
+
             await self._execute_request(action='upload', path=urn.quote(), data=file_sender(buffer))
         else:
             await self._execute_request(action='upload', path=urn.quote(), data=buffer)
@@ -1016,7 +1035,7 @@ class Client(object):
     async def get_property(
         self,
         path: Union[str, "os.PathLike[str]"],
-        option: dict[str, str]
+        option: Dict[str, str]
     ) -> Union[str, None]:
         """
         Gets metadata property of remote resource on WebDAV server.
@@ -1026,7 +1045,7 @@ class Client(object):
             path (``str``):
                 Path to remote directory.
 
-            option (``dict[str, str]``): the property attribute as dictionary with following keys:
+            option (``Dict[str, str]``): the property attribute as dictionary with following keys:
                     `namespace`: (optional) the namespace for XML property which will be set,
                     `name`: the name of property which will be set.
 
@@ -1046,7 +1065,7 @@ class Client(object):
     async def set_property(
         self,
         path: Union[str, "os.PathLike[str]"],
-        option: dict[str, str]
+        option: Dict[str, str]
     ) -> None:
         """
         Sets metadata property of remote resource on WebDAV server.
@@ -1056,7 +1075,7 @@ class Client(object):
             path (``str``):
                 Path to remote directory.
 
-            option (``dict[str, str]``): the property attribute as dictionary with following keys:
+            option (``Dict[str, str]``): the property attribute as dictionary with following keys:
                     `namespace`: (optional) the namespace for XML property which will be set,
                     `name`: the name of property which will be set,
                     `value`: (optional) the value of property which will be set. Defaults is empty string.
@@ -1093,4 +1112,126 @@ class Client(object):
         """
 
         urn = Urn(path)
-        return Resource(self, urn)  
\ No newline at end of file
+        return Resource(self, urn)  
+
+class Resource(object):
+    """
+    Remote resource.
+    """
+
+    def __init__(
+        self, 
+        client: Client, 
+        urn: Urn
+    ) -> None:
+        self.client = client
+        self.urn = urn
+
+    def __str__(self) -> str:
+        return f"resource {self.urn.path()}"
+
+    async def is_directory(self) -> bool:
+        """
+        Determine if the resource is a directory.
+
+        Returns:
+            :obj:`bool`: True if the resource is a directory or False else.
+        """
+        return await self.client.is_directory(self.urn.path())
+
+    async def rename(
+        self, 
+        name: str
+    ) -> None:
+        """
+        Rename the resource.
+
+        Parameters:    
+            name (``str``):
+                New name to resource.
+        """
+        old_path = self.urn.path()
+        parent_path = self.urn.parent()
+        name = Urn(name).filename()
+        new_path = f"{parent_path}{name}"
+
+        await self.client.move(source=old_path, destination=new_path)
+        self.urn = Urn(new_path)
+
+    async def move(
+        self, 
+        path: Union[str, "os.PathLike[str]"]
+    ) -> None:
+        """
+        Move the resource to new path.
+
+        Parameters:    
+            path (``str``):
+                New path of the resource.
+        """
+        new_urn = Urn(path)
+        await self.client.move(source=self.urn.path(), destination=new_urn.path())
+        self.urn = new_urn
+
+    async def copy(
+        self,
+        path: Union[str, "os.PathLike[str]"]
+    ) -> "Resource":
+        """
+        Copy the resource to a another path.
+
+        Parameters:    
+            path (``str``):
+                The path where resource will be copied.
+
+        Returns:
+            :obj:`Resource`: The value of property or None if property is not found.
+        """
+        urn = Urn(path)
+        await self.client.copy(source=self.urn.path(), destination=path)
+        return Resource(self.client, urn)
+
+    async def info(
+        self, 
+        filter: Optional[Iterable[str]] = None
+    ) -> Dict[str, str]:
+        """
+        Get a dictionary with resource information.
+
+        Parameters:    
+            filter (``Iterable[str]``, *optional*):
+                If filter is not `None` then only return properties
+                contained in filter iterable.
+
+        Returns:
+            :obj:`Dict[str, str]`: Information about the resource
+        """
+
+        info = await self.client.info(self.urn.path())
+        if not filter:
+            return info
+        return {key: value for (key, value) in info.items() if key in filter}
+
+    async def unlink(self) -> None:
+        """
+        Delete the resource.
+        """
+
+        await self.client.unlink(self.urn.path())
+
+    async def delete(self):
+        """
+        Delete the resource.
+        """
+
+        await self.unlink()
+
+    async def exists(self) -> bool:
+        """
+        Determine if the resource exists in the remote WebDAV server
+
+        Returns:
+            :obj:`bool`: True if the resource exists else return False.
+        """
+
+        return await self.client.exists(self.urn.path())
\ No newline at end of file
